from pathlib import Path
from typing import List

import pandoc
from docutils import nodes
from docutils.core import publish_doctree
from robot.running import TestCase
from robot.testdoc import JsonConverter, TestSuiteFactory
from sphinx.util.docutils import SphinxDirective


class SphinxJsonConverter(JsonConverter):
    def _convert_test(self, test: TestCase):
        ret_test = super()._convert_test(test)
        ret_test['lineno'] = test.lineno
        ret_test['source'] = test.source
        return ret_test


class RfTestDoc(SphinxDirective):
    """A directive for RF TestDocs!"""
    required_arguments = 1

    def handle_test(self, test: dict) -> nodes.Node:
        section_id = nodes.make_id(test.get('fullName'))
        test_section = nodes.section(ids=[section_id])
        test_section += nodes.title(text=test.get('name'))

        if test.get('lineno') and test.get('source'):
            source: Path = test['source']
            line_no = test['lineno']
            relative_src = source.relative_to(self.config.rf_test_doc_root)
            link_url = f"{self.config.rf_test_doc_root_url}/{relative_src}#L{line_no}"
            ln_link = nodes.reference(refuri=link_url, text="Test Case source")
            ln_link["target"] = "_blank"  # or ln_link["classes"] = ["external"]
            ln_para = nodes.paragraph()
            ln_para += ln_link
            test_section += ln_para

        if test.get('doc'):
            section_id = nodes.make_id(f"{test.get('fullName')} test doc")
            doc_section = nodes.section(ids=[section_id])
            doc_section += nodes.title(text="Test Documentation")
            doc_pan = pandoc.read(source=test.get('doc'), format="html")
            doc_rst = pandoc.write(doc=doc_pan, format="rst")
            doc_tree = publish_doctree(doc_rst)
            doc_section.extend(doc_tree.children)
            test_section += doc_section

        return test_section

    def handle_suite(self, suite: dict) -> nodes.Node:
        section_id = nodes.make_id(suite.get('fullName'))
        suite_section = nodes.section(ids=[section_id])
        suite_section += nodes.title(text=suite.get('name'))

        if suite.get('doc'):
            section_id = nodes.make_id(f"{suite.get('fullName')} suite doc")
            doc_section = nodes.section(ids=[section_id])
            doc_section += nodes.title(text="Suite Documentation")
            doc_pan = pandoc.read(source=suite.get('doc'), format="html")
            doc_rst = pandoc.write(doc=doc_pan, format="rst")
            doc_tree = publish_doctree(doc_rst)
            doc_section.extend(doc_tree.children)
            suite_section += doc_section

        suite_section += nodes.transition()
        section_id = nodes.make_id(f"{suite.get('fullName')} test cases")
        test_section = nodes.section(ids=[section_id])
        test_section += nodes.title(text="Test Cases")
        for child_test in suite.get('tests', []):
            test_section += self.handle_test(child_test)
        suite_section += test_section

        suite_section += nodes.transition()
        section_id = nodes.make_id(f"{suite.get('fullName')} Child Suites")
        child_suite_section = nodes.section(ids=[section_id])
        child_suite_section += nodes.title(text="Child Suites")
        for child_suite in suite.get('suites', []):
            child_suite_section += self.handle_suite(child_suite)
        suite_section += child_suite_section

        return suite_section

    def run(self) -> List[nodes.Node]:
        suite_root = Path(self.config.rf_test_doc_root) / self.arguments[0]
        raw_suite = TestSuiteFactory([suite_root])
        suite = SphinxJsonConverter().convert(raw_suite)
        return [self.handle_suite(suite)]
